
'Software License Agreement and Disclaimers                                        
'                                                                   
'THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,  
'WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED  
'TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A       
'PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,  
'IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR         
'CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER. ABSOLUTELY NO TECHNICAL 
'SUPPORT WILL BE PROVIDE IN ANY REGARDS TO THIS SOFTWARE

'Information furnished by Maxim Integrated Products and Dallas Semiconductor 
'is believed to be accurate and reliable. However, the company cannot assume 
'responsibility for use of anything other than the circuitry entirely embodied 
'in a Maxim product; nor for any infringements of patents or other rights of third 
'parties that may result from this software use. No license is granted by implication or 
'otherwise under any patent rights of Maxim Integrated Products. Maxim reserves the 
'right to change the software without notice. 

'Copyright Maxim Integrated Products 2006

'Author: Craig Lyon
'Company: Maxim Integrated Products
'Date: Novemeber 18, 2006
'Platform: Microsoft Visual Studio 2005, Visual Basic Express Edition
'Revision: 1.0


'Import Math Functions
Imports system.math


Public Class Main

#Region "Variable Declarations"
    'Definition of Public Variables within Main Class

    'Declaration of available thermistors. The Resistor value (at 25C) is listed in the
    'Thermistor array, the beta values (for each corresponding resistor value) is listed
    'in the Betas array.
    'The values were taken from commonly available thermistors. More values can be added
    'or others can be deleted. The values should be in ascending order and the
    'index of the resistance must be the same index of the beta.

    Dim Thermistors() As Integer = {22, 33, 40, 47, 68, 100, 150, 1000, 1500, 2000, _
                                            2200, 3000, 3300, 4700, 6800, 10000, 15000, 22000, _
                                            33000, 47000, 68000, 100000, 150000, 220000, 330000, _
                                            470000}

    Dim Betas() As Integer = {2750, 2750, 2750, 2750, 2800, 2800, 2800, 4500, 4500, 4500, _
                                    4500, 4500, 4500, 4500, 4250, 4250, 4250, 4250, _
                                    4250, 4700, 4700, 4700, 4700, 4700, 4700, _
                                    4700}

    '1% resistor value declarations
    Dim Res1per() As Double = {10.0, 10.2, 10.5, 10.7, 11.0, 11.3, 11.5, 11.8, 12.1, 12.4, 12.7, 13.0, _
                                13.3, 13.7, 14.0, 14.3, 14.7, 15.0, 15.4, 15.8, 16.2, 16.5, 16.9, 17.4, _
                                17.8, 18.2, 18.7, 19.1, 19.6, 20.0, 20.5, 21.0, 21.5, 22.1, 22.6, 23.2, _
                                23.7, 24.3, 24.9, 25.5, 26.1, 26.7, 27.4, 28.0, 28.7, 29.4, 30.1, 30.9, _
                                31.6, 32.4, 33.2, 34.0, 34.8, 35.7, 36.5, 37.4, 38.3, 39.2, 40.2, 41.2, _
                                42.2, 43.2, 44.2, 45.3, 46.4, 47.5, 48.7, 49.9, 51.1, 52.3, 53.6, 54.9, _
                                56.2, 57.6, 59.0, 60.4, 61.9, 63.4, 64.9, 66.5, 68.1, 69.8, 71.5, 73.2, _
                                75.0, 76.8, 78.7, 80.6, 82.5, 84.5, 86.6, 88.7, 90.9, 93.1, 95.3, 97.6}
    '5% resistor value declarations
    Dim Res5per() As Double = {10, 11, 12, 13, 15, 16, 18, 20, 22, 24, 27, 30, 33, 36, 39, 43, 47, 51, 56, 62, 68, 75, 82, 91}

    'Range of available values. The Range array will scale the 1% and 5% resistor arrays to cover
    'all available resistor values. Note: A very large value and a very small value are added to 
    'indicate opens and shorts
    Dim Range() As Double = {0.0000000001, 0.1, 1, 10, 100, 1000, 10000, 100000, 10000000000.0}

    'Declaration of the population size
    'One hundred units with up to 16 variables
    Dim Population(100, 16) As Double

    Dim Upperbound As Integer

    'Hold the final resistor values (1% or 5%)
    Dim Resistors() As Double

    Dim ResetCount As Integer

    Dim Rvalues(16) As Double

    'Calculated output values for each of the population
    'Output is calculated at up to 125 points
    Public Output(100, 126) As Double

    'Boolean variable used to determine if the user has requested to stop
    'the algorithm
    Dim contin As Boolean

    'Boolean variable used to determine if 1% or 5% resistors are to be used
    'in the calculations
    Dim R5Per As Boolean = True

    Dim Vin As Double

    'Variable that holds the number of resistor elements (x2) for each population
    Dim CRN As Integer

    'Array that holds the desired resistance/voltage across the temeperature range
    Public Input(126) As Double

    Public MaxTemp As Integer
    Public MinTemp As Integer

#End Region

#Region "Functions / SubRoutines"


    'This functions initializes the graph and starts the genetic algorithm process
    Private Sub Run_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Run.Click

        'Initialize and show the form that contains the graph
        graph.InitializeLifetimeService()
        graph.Show()

        contin = True

        'Initialize the Pseudo Random number generator
        Randomize()

        'Disable the run button, the user must push the stop button before pressing
        'the run button again
        Run.Enabled = False

        MinTempB.Enabled = False
        MaxTempB.Enabled = False
        CoA.Enabled = False
        CoX.Enabled = False
        CoX2.Enabled = False
        CoX3.Enabled = False
        CoX4.Enabled = False
        Configuration.Enabled = False
        InV1.Enabled = False
        CheckBox1.Enabled = False
        CheckBox2.Enabled = False
        CALINPUT.Enabled = False


        Dim HoldPop(5) As Integer

        Dim RSCount As Integer
        Dim MinError As Double

        Dim k As Integer
        Dim i As Integer
        Dim x As Integer
        Dim pass As Integer
        Dim Comp As Integer

        Dim Shift As Integer

        Dim N As Integer

        Dim Mix1 As Integer
        Dim Mix2 As Integer

        Dim hold(16) As Double

        Dim HoldErr As Double
        Dim HoldErr2 As Double

        'Holds a newely generated element of a population
        Dim HoldV(16) As Integer

        'Array that holds the amount of error for each population unit
        Dim HoldOutError(100) As Double

        'Variable that holds the Va voltage
        'Set to zero if left blank
        If InV1.Text = "" Then
            InV1.Text = "0"
        End If

        Vin = CDbl(InV1.Text)

        'Determine if 5% or 1% resistor values are to be used and
        'set the upperbound and resistor array accordingly
        If R5Per = True Then
            Upperbound = UBound(Res5per)
            ReDim Resistors(Upperbound)

            'Set resistors to 5% Values
            For k = 0 To UBound(Resistors)
                Resistors(k) = Res5per(k)
            Next
        Else
            Upperbound = UBound(Res1per)
            ReDim Resistors(Upperbound)

            'Set Resitors to 1% Values
            For k = 0 To UBound(Resistors)
                Resistors(k) = Res1per(k)
            Next
        End If

        'Set the CRN variable according to the selected
        'Thermistor / Resistor Configuration
        'The CRN variable = 2*Resistors (one element for resistance and one for range)
        Select Case Configuration.SelectedIndex
            Case 0
                CRN = 6
            Case 1
                CRN = 6
            Case 2
                CRN = 8
            Case 3
                CRN = 12
        End Select

        'Reset the RSCount and Error Value
        RSCount = 0
        MinError = 10

        'Calculate the minimum and maximum temperatures based on the
        'values selected by the user
        MinTemp = 5 * MinTempB.SelectedIndex
        MaxTemp = (30 + 5 * MaxTempB.SelectedIndex) + 40

        'Set the coefficients of the array matrix to zero
        'for any that have no input
        If CoX3.Text = "" Then
            CoX3.Text = "0"
        End If

        If CoX2.Text = "" Then
            CoX2.Text = "0"
        End If

        If CoX.Text = "" Then
            CoX.Text = "0"
        End If

        If CoA.Text = "" Then
            CoA.Text = "0"
        End If

        'Calculate the Desired Resistance / Voltage Values over the full temperature range
        For k = MinTemp To MaxTemp
            x = k - 40
            Input(k) = CDbl(CoX3.Text) * (x * x * x) + CDbl(CoX2.Text) * (x * x) + CDbl(CoX.Text) * x + CDbl(CoA.Text)

        Next



        'Begining of the Genetic Algorithm
        'Note there are many ways to implement a Genetic Algorithm. The example below
        'is just an example is a simplified and not a fully optimized algorithm for 
        'speed of convergence

        'Declare 100 Random Population elements
        For k = 0 To UBound(Population) - 1

            'Create a random element of a population
            HoldV = RandPop()

            'Store the new element in the population
            For i = 0 To 15
                Population(k, i) = HoldV(i)
            Next

        Next k

        N = 0

whileloop:
        'While the user has not requested that the algorithm be stopped
        While contin = True

            'Calculate the output of each population element
            For k = N To UBound(Population) - 1
                CalOutput(k)
            Next

            'Calculate the error for each population element
            'and determine the best one
            For k = N To UBound(Population) - 1

                HoldErr = 0

                For i = MinTemp To MaxTemp
                    'sum the squared error of each temperature point
                    HoldErr2 = Abs((Output(k, i) - Input(i)))
                    HoldErr = HoldErr + HoldErr2 * HoldErr2
                Next

                'Divide by the number of temperature elements and set the
                'error for the given population element
                Population(k, 15) = (HoldErr / (MaxTemp - MinTemp))

            Next

            'Sort the population based on error
            'Simple bubble sort loop
            For pass = 1 To (UBound(Population) - 2)
                For Comp = 1 To (UBound(Population) - 2)
                    If Population(Comp, 15) > Population(Comp + 1, 15) Then
                        For k = 0 To 15
                            hold(k) = Population(Comp, k)
                            Population(Comp, k) = Population(Comp + 1, k)
                            Population(Comp + 1, k) = hold(k)
                        Next k
                    End If
                Next
            Next pass

            'The population elements are now sorted from 1 to 100 in order of lowest
            'error to largest error

            'Natural selection is introduced by eliminating the worst 20
            'A new population element (child) from each of the best results is then created
            'The best 4 are retained through the next generation
            For k = 4 To 80

                'Create a random variable from 1 to 14
                i = Int((13 + 1) * Rnd()) + 1

                'Do a random mix of the population based on the best
                '20 from the population
                'Note that the mix does not have to be random, it can be mixed
                'in a variety of ways with different convergence speeds
                For i = 1 To i + 1
                    Mix1 = Int((13 + 1) * Rnd())
                    Mix2 = Int((15 + 1) * Rnd()) + 1

                    Population(k, Mix1) = Population(Mix2, Mix1)
                Next

            Next

            'Create additional random population to fill 80 through 100
            For k = 81 To UBound(Population) - 1

                'Create a random element of a population
                HoldV = RandPop()

                'Store the new element in the population
                For i = 0 To 15
                    Population(k, i) = HoldV(i)
                Next

            Next

            'Set N to 4, you don't need to recalculate the first 4
            'The error is maintained through the sort algorithm
            N = 4

            'If this is the best one yet, do the following
            If Population(1, 15) < Population(0, 15) Then
                'Set the zero position to the best set of values
                For k = 0 To 15
                    Population(0, k) = Population(1, k)
                Next
                'Reset the shift count
                Shift = 0

                ResetCount = 0

                'Calculate the output for the new best solution
                CalOutput(0)

                'ReDraw the graph with the new best solution
                graph.Invalidate()

                'Update the count and error values
                MeanSqError.Text = Format(Population(0, 15), "0.00E+00")
                Iteration.Text = RSCount

                'Update the list box to display the Thermistor / Resistor Values
                Call OutResults()

            End If

            'Every 10 iterations do the following
            If RSCount Mod 10 = 0 Then

                'If the first couple population elements are identical, it is likely that the 
                'population has become saturated. To eliminate this problem the first elements
                'could have mutations integrated into their array elements or the population can be
                'reset to start searching from a different intial condition
                If Population(1, 15) = Population(2, 15) And Population(1, 15) = Population(3, 15) Then
                    ResetCount = ResetCount + 1
                End If

                'If the first few elements are the same for 1000 iterations (10* 100)
                'Reset the population
                If ResetCount > 100 Then

                    Randomize()

                    For k = 1 To UBound(Population) - 1

                        'Create a random element of a population
                        HoldV = RandPop()

                        'Store the new element in the population
                        For i = 0 To 15
                            Population(k, i) = HoldV(i)
                        Next
                    Next

                    N = 1
                    ResetCount = 0

                    GoTo WhileLoop

                End If

                'Update the iteration count and error count
                Iteration.Text = RSCount
                MeanSqError.Text = Format(Population(1, 15), "0.00E+00")

                System.Windows.Forms.Application.DoEvents()

                'keep track of the number of shifts that occur
                Shift = Shift + 1

                'If there are more than 10 shifts, re-randomize and and reset all
                'population elements greater than 4
                If Shift = 10 Then
                    Shift = 0
                    Randomize()

                    For k = 4 To UBound(Population) - 1

                        'Create a random element of a population
                        HoldV = RandPop()

                        'Store the new element in the population
                        For i = 0 To 15
                            Population(k, i) = HoldV(i)
                        Next

                    Next

                Else
                    'Shift the population elements (To search for a minimum)
                    N = 3 + 2 * CRN
                    'shift all starting with population element #2
                    Call ShiftPop(2, 2 * CRN + 2)
                End If

                'randomize any remaing population elements (after the shift
                For k = N To UBound(Population) - 1

                    HoldV = RandPop()

                    For i = 0 To 14
                        Population(k, i) = HoldV(i)
                    Next

                Next

            End If

            'Iteration Counter
            RSCount = RSCount + 1

        End While

    End Sub

    'This function creates a new random population given the selected
    'configuration of the resistor / thermistor network
    Private Function RandPop() As Integer()

        Dim RandV(16) As Integer

        Dim k As Integer

        'Create random selection of thermisto values
        Select Case Configuration.SelectedIndex
            Case 0
                RandV(CRN) = CInt(Int((UBound(Thermistors) + 1) * Rnd()))
            Case 1
                RandV(CRN) = CInt(Int((UBound(Thermistors) + 1) * Rnd()))
            Case 2
                RandV(CRN) = CInt(Int((UBound(Thermistors) + 1) * Rnd()))
                RandV(CRN + 1) = CInt(Int((UBound(Thermistors) + 1) * Rnd()))
            Case 3
                RandV(CRN) = CInt(Int((UBound(Thermistors) + 1) * Rnd()))
                RandV(CRN + 1) = CInt(Int((UBound(Thermistors) + 1) * Rnd()))
                RandV(CRN + 2) = CInt(Int((UBound(Thermistors) + 1) * Rnd()))
        End Select

        'Create random selection of resitor values
        For k = 0 To CRN - 1 Step 2
            RandV(k) = CInt(Int((Upperbound + 1) * Rnd()))
            RandV(k + 1) = CInt(Int((UBound(Range) + 1) * Rnd()))
        Next

        Return RandV

    End Function

    'This function is used to search for minimums near the random minimums generated
    'by the genetic algorithm and is intended to speed the convergence of the 
    'algorithm. Note: This will not speed the convergence in all cases.

    'The function basically just shifts each of the populations variables up or down by one
    Private Sub ShiftPop(ByVal LowB As Integer, ByVal HighB As Integer)

        Dim k As Integer
        Dim i As Integer
        Dim L As Integer

        Dim MaxR As Integer
        Dim MaxT As Integer

        MaxT = UBound(Thermistors)

        If R5Per = True Then
            MaxR = UBound(Res5per)
        Else
            MaxR = UBound(Res1per)
        End If

        i = 0
        L = 1

        'The following loops through a population element and then shifts
        'one of its values up or down
        For k = LowB To HighB

            If Population(k, i) + L >= MaxR Or Population(k, i) + L < 0 Then
                If L = 1 Then
                    Population(k, i) = 0
                Else
                    Population(k, i) = MaxR
                End If
            Else
                Population(k, i) = Population(k, i) + L
            End If

            i = i + 2

            If i > CRN - 1 Then
                i = 0
                If L = 1 Then
                    L = -1
                Else
                    L = 1
                End If
            End If
        Next k

    End Sub

    'This function calculates the output values for a given input
    'The results are used for graph and error calculations
    Private Sub CalOutput(ByVal Indx)

        Dim R1 As Double
        Dim R2 As Double
        Dim R3 As Double
        Dim R4 As Double
        Dim R5 As Double
        Dim R6 As Double
        Dim R7 As Double
        Dim R8 As Double
        Dim R9 As Double

        Dim R1H As Double
        Dim R2H As Double
        Dim R3H As Double

        Dim I As Integer


        If R5Per = True Then

        End If

        R1 = Resistors(Population(Indx, 0)) * Range(Population(Indx, 1))
        R2 = Resistors(Population(Indx, 2)) * Range(Population(Indx, 3))
        R3 = Resistors(Population(Indx, 4)) * Range(Population(Indx, 5))


        If CALINPUT.SelectedIndex = 0 Then 'Calculate the resistance from point A to GND
            Select Case Configuration.SelectedIndex
                Case 0
                    For I = MinTemp To MaxTemp

                        'Thermistor resistance vs. Temperature calculation, based on kelvin scale
                        R4 = Thermistors(Population(Indx, 6)) * Exp(Betas(Population(Indx, 6)) * ((1 / (233.15 + I)) - (1 / 298.15)))

                        R1H = 1 / ((1 / R1) + (1 / R4)) + R2
                        R2H = R3

                        Output(Indx, I) = R1H + R2H

                    Next
                Case 1
                    For I = MinTemp To MaxTemp
                        R4 = Thermistors(Population(Indx, 6)) * Exp(Betas(Population(Indx, 6)) * ((1 / (233.15 + I)) - (1 / 298.15)))

                        R1H = R1
                        R2H = 1 / ((1 / R3) + (1 / R4)) + R2

                        Output(Indx, I) = R1H + R2H

                    Next
                Case 2
                    For I = MinTemp To MaxTemp
                        R4 = Resistors(Population(Indx, 6)) * Range(Population(Indx, 7))
                        R5 = Thermistors(Population(Indx, 8)) * Exp(Betas(Population(Indx, 8)) * ((1 / (233.15 + I)) - (1 / 298.15)))
                        R6 = Thermistors(Population(Indx, 9)) * Exp(Betas(Population(Indx, 9)) * ((1 / (233.15 + I)) - (1 / 298.15)))

                        R1H = 1 / ((1 / R1) + (1 / R5)) + R2
                        R2H = 1 / ((1 / R3) + (1 / R6)) + R4

                        Output(Indx, I) = R1H + R2H

                    Next

                Case 3
                    For I = MinTemp To MaxTemp
                        R4 = Resistors(Population(Indx, 6)) * Range(Population(Indx, 7))
                        R5 = Resistors(Population(Indx, 8)) * Range(Population(Indx, 9))
                        R6 = Resistors(Population(Indx, 10)) * Range(Population(Indx, 11))
                        R7 = Thermistors(Population(Indx, 12)) * Exp(Betas(Population(Indx, 12)) * ((1 / (233.15 + I)) - (1 / 298.15)))
                        R8 = Thermistors(Population(Indx, 13)) * Exp(Betas(Population(Indx, 13)) * ((1 / (233.15 + I)) - (1 / 298.15)))
                        R9 = Thermistors(Population(Indx, 14)) * Exp(Betas(Population(Indx, 14)) * ((1 / (233.15 + I)) - (1 / 298.15)))

                        R1H = 1 / ((1 / R1) + (1 / R7)) + R2
                        R2H = 1 / ((1 / R3) + (1 / R8)) + R4
                        R3H = 1 / ((1 / R5) + (1 / R9)) + R6

                        Output(Indx, I) = R1H + R2H + R3H

                    Next

            End Select
        Else ' Calculate the Voltage at point B to GND given the Voltage at Point A to GND
            Select Case Configuration.SelectedIndex
                Case 0
                    For I = MinTemp To MaxTemp

                        R4 = Thermistors(Population(Indx, 6)) * Exp(Betas(Population(Indx, 6)) * ((1 / (233.15 + I)) - (1 / 298.15)))

                        R1H = 1 / ((1 / R1) + (1 / R4)) + R2
                        R2H = R3

                        Output(Indx, I) = (Vin * ((R2H) / (R1H + R2H)))

                    Next
                Case 1
                    For I = MinTemp To MaxTemp
                        R4 = Thermistors(Population(Indx, 6)) * Exp(Betas(Population(Indx, 6)) * ((1 / (233.15 + I)) - (1 / 298.15)))

                        R1H = R1
                        R2H = 1 / ((1 / R3) + (1 / R4)) + R2

                        Output(Indx, I) = (Vin * ((R2H) / (R1H + R2H)))

                    Next
                Case 2
                    For I = MinTemp To MaxTemp
                        R4 = Resistors(Population(Indx, 6)) * Range(Population(Indx, 7))
                        R5 = Thermistors(Population(Indx, 8)) * Exp(Betas(Population(Indx, 8)) * ((1 / (233.15 + I)) - (1 / 298.15)))
                        R6 = Thermistors(Population(Indx, 9)) * Exp(Betas(Population(Indx, 9)) * ((1 / (233.15 + I)) - (1 / 298.15)))

                        R1H = 1 / ((1 / R1) + (1 / R5)) + R2
                        R2H = 1 / ((1 / R3) + (1 / R6)) + R4

                        Output(Indx, I) = (Vin * ((R2H) / (R1H + R2H)))

                    Next

                Case 3
                    For I = MinTemp To MaxTemp
                        R4 = Resistors(Population(Indx, 6)) * Range(Population(Indx, 7))
                        R5 = Resistors(Population(Indx, 8)) * Range(Population(Indx, 9))
                        R6 = Resistors(Population(Indx, 10)) * Range(Population(Indx, 11))
                        R7 = Thermistors(Population(Indx, 12)) * Exp(Betas(Population(Indx, 12)) * ((1 / (233.15 + I)) - (1 / 298.15)))
                        R8 = Thermistors(Population(Indx, 13)) * Exp(Betas(Population(Indx, 13)) * ((1 / (233.15 + I)) - (1 / 298.15)))
                        R9 = Thermistors(Population(Indx, 14)) * Exp(Betas(Population(Indx, 14)) * ((1 / (233.15 + I)) - (1 / 298.15)))

                        R1H = 1 / ((1 / R1) + (1 / R7)) + R2
                        R2H = 1 / ((1 / R3) + (1 / R8)) + R4
                        R3H = 1 / ((1 / R5) + (1 / R9)) + R6

                        Output(Indx, I) = (Vin * ((R2H + R3H) / (R1H + R2H + R3H)))

                    Next

            End Select

        End If


    End Sub

    'This function write the resistor / thermistor values to the output list box
    Private Sub OutResults()

        Results.Items.Clear()

        Dim Hold As String
        Dim Hold2 As String


        If R5Per = True Then

        End If

        Results.Items.Add("Error =" + Format(Population(0, 15), "0.00E+00"))
        Hold = CheckVal(Resistors(Population(0, 0)) * Range(Population(0, 1)))
        Results.Items.Add("R1 = " + Hold)
        Hold = CheckVal(Resistors(Population(0, 2)) * Range(Population(0, 3)))
        Results.Items.Add("R2 = " + Hold)
        Hold = CheckVal(Resistors(Population(0, 4)) * Range(Population(0, 5)))
        Results.Items.Add("R3 = " + Hold)

        Select Case Configuration.SelectedIndex
            Case 0

                Hold = CStr(Thermistors(Population(0, 6)))
                Hold2 = CStr(Betas(Population(0, 6)))
                Results.Items.Add("T1 = " + Hold + "  Beta = " + Hold2)
            Case 1
                Hold = CStr(Thermistors(Population(0, 6)))
                Hold2 = CStr(Betas(Population(0, 6)))
                Results.Items.Add("T1 = " + Hold + "  Beta = " + Hold2)

            Case 2
                Hold = CheckVal(Resistors(Population(0, 6)) * Range(Population(0, 7)))
                Results.Items.Add("R4 = " + Hold)
                Hold = CStr(Thermistors(Population(0, 8)))
                Hold2 = CStr(Betas(Population(0, 8)))
                Results.Items.Add("T1 = " + Hold + "  Beta = " + Hold2)
                Hold = CStr(Thermistors(Population(0, 9)))
                Hold2 = CStr(Betas(Population(0, 9)))
                Results.Items.Add("T2 = " + Hold + "  Beta = " + Hold2)

            Case 3
                Hold = CheckVal(Resistors(Population(0, 6)) * Range(Population(0, 7)))
                Results.Items.Add("R4 = " + Hold)
                Hold = CheckVal(Resistors(Population(0, 8)) * Range(Population(0, 9)))
                Results.Items.Add("R5 = " + Hold)
                Hold = CheckVal(Resistors(Population(0, 10)) * Range(Population(0, 11)))
                Results.Items.Add("R6 = " + Hold)
                Hold = CStr(Thermistors(Population(0, 12)))
                Hold2 = CStr(Betas(Population(0, 12)))
                Results.Items.Add("T1 = " + Hold + "  Beta = " + Hold2)
                Hold = CStr(Thermistors(Population(0, 13)))
                Hold2 = CStr(Betas(Population(0, 13)))
                Results.Items.Add("T2 = " + Hold + "  Beta = " + Hold2)
                Hold = CStr(Thermistors(Population(0, 14)))
                Hold2 = CStr(Betas(Population(0, 14)))
                Results.Items.Add("T3 = " + Hold + "  Beta = " + Hold2)


        End Select

        System.Windows.Forms.Application.DoEvents()

    End Sub

    'Formating function that changes the text to open or 0 for large and small
    'resistor values respectively
    Private Function CheckVal(ByVal A As Double) As String

        If A < 0 Then
            CheckVal = "0.0"
            Exit Function
        End If

        If A > 100000000.0 Then
            CheckVal = "Open"
            Exit Function
        End If

        CheckVal = Format(A, "Fixed")

    End Function

    'Stop Button
    Private Sub StopIt_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles StopIt.Click
        contin = False
        Run.Enabled = True

        MinTempB.Enabled = True
        MaxTempB.Enabled = True
        CoA.Enabled = True
        CoX.Enabled = True
        CoX2.Enabled = True
        CoX3.Enabled = True
        CoX4.Enabled = True
        Configuration.Enabled = True
        InV1.Enabled = True
        CheckBox1.Enabled = True
        CheckBox2.Enabled = True
        CALINPUT.Enabled = True

    End Sub

    '1% or 5% resistor selection
    Private Sub CheckBox1_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CheckBox1.CheckedChanged
        If CheckBox1.Checked = True Then
            R5Per = True
            CheckBox2.Checked = False
        Else
            CheckBox2.Checked = True
            R5Per = False
        End If
    End Sub

    Private Sub CheckBox2_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CheckBox2.CheckedChanged
        If CheckBox2.Checked = True Then
            CheckBox1.Checked = False
        Else
            CheckBox1.Checked = True
        End If
    End Sub

    Private Sub Main_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        MinTempB.SelectedIndex = 0
        Configuration.SelectedIndex = 2
        CALINPUT.SelectedIndex = 1
        MaxTempB.SelectedIndex = MaxTempB.Items.Count - 1
        PictureBox1.Visible = True
    End Sub
    

   

    'Show a diagram of the selected configuration   
    Private Sub Configuration_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Configuration.SelectedIndexChanged
        Select Case Configuration.SelectedIndex
            Case 0
                PictureBox1.Visible = True
                PictureBox2.Visible = False
                PictureBox3.Visible = False
                PictureBox4.Visible = False

            Case 1
                PictureBox1.Visible = False
                PictureBox2.Visible = True
                PictureBox3.Visible = False
                PictureBox4.Visible = False

            Case 2
                PictureBox1.Visible = False
                PictureBox2.Visible = False
                PictureBox3.Visible = True
                PictureBox4.Visible = False

            Case 3
                PictureBox1.Visible = False
                PictureBox2.Visible = False
                PictureBox3.Visible = False
                PictureBox4.Visible = True
        End Select

    End Sub

    Private Sub Button1_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button1.Click
        contin = False
        Run.Enabled = True

        MinTempB.Enabled = True
        MaxTempB.Enabled = True
        CoA.Enabled = True
        CoX.Enabled = True
        CoX2.Enabled = True
        CoX3.Enabled = True
        CoX4.Enabled = True
        Configuration.Enabled = True
        InV1.Enabled = True
        CheckBox1.Enabled = True
        CheckBox2.Enabled = True
        CALINPUT.Enabled = True

        Me.Close()

    End Sub


#End Region

    
End Class

