

Public Class graph
    Dim RcDraw As Rectangle

    'The Graphs Region relates only to functions and methods used to draw the graphs
    'used for the visual representation of the GA convergence. The code below
    'is not part of the genetic algorithm.

#Region "Graphs"

    Private Sub Results_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load

        If Main.CALINPUT.SelectedIndex = 0 Then
            Label1.Text = "Resistance vs. Temperature"
        Else
            Label1.Text = "Voltage  vs. Temperature"
        End If

        'ReDraw the graph
        Me.Invalidate()

    End Sub

    'This function scales and draws the graph on the form along with the traces.
    Public Sub DrawCurvePoint(ByVal sender As Object, ByVal e As System.Windows.Forms.PaintEventArgs) Handles MyBase.Paint

        Dim O As Integer '(Offset)
        Dim H As Integer 'Height
        Dim W As Integer 'Width

        Dim InputG() As Double
        Dim OutputG() As Double

        Dim VScale As Double
        Dim HScale As Double
        Dim Hold As Integer
        Dim Max As Double
        Dim Min As Double
        Dim VOffset As Double

        Dim GraphText As String

        ' Create pens.
        Dim redPen As New Pen(Color.Red, 2)
        Dim greenPen As New Pen(Color.Green, 2)
        Dim grayPen As New Pen(Color.Gray, 0.5)
        Dim bluePen As New Pen(Color.Blue, 3)

        ' Create points that define the graph size.
        Dim point1 As New Point(O, O)
        Dim point2 As New Point(H + O, O)
        Dim point3 As New Point(H + O, W + O)
        Dim point4 As New Point(O, W + O)
        Dim point5 As New Point(O, O)

        Dim points() As Point


        Dim k As Integer

        'Try

        O = 75
        H = 325
        W = 325
        Hold = UBound(Main.Input)

        ReDim InputG(Hold)
        ReDim OutputG(Hold)

        'Set the background of the Graph to white
        e.Graphics.FillRectangle(Brushes.White, O, O, H, W)


        'Create the grid
        ReDim points(1)

        'Grid Points
        For k = 0 To 10
            points(0) = (New Point(k * H / 10 + O, O))
            points(1) = (New Point(k * H / 10 + O, W + O))
            e.Graphics.DrawLines(grayPen, points)
            points(0) = (New Point(O, k * W / 10 + O))
            points(1) = (New Point(H + O, k * W / 10 + O))
            e.Graphics.DrawLines(grayPen, points)
        Next


        'Get the maximum and minimum temperatures
        Max = Main.Input(Main.MinTemp)
        Min = Main.Input(Main.MinTemp)

        'Find the Maximum and Minimum vertical scale values
        For k = Main.MinTemp To Main.MaxTemp
            If Main.Input(k) > Max Then
                Max = Main.Input(k)
            End If
            If Main.Input(k) < Min Then
                Min = Main.Input(k)
            End If
        Next

        'Apply scaling factors to fit the graph

        'Vertical Scale Scaling / Offsets
        VScale = H / (1.3 * (Max - Min))
        VOffset = VScale * Min - 0.1 * H

        'Horizontal Scale Scaling / Offsets
        HScale = W / (Main.MaxTemp - Main.MinTemp)

        'Set the number of points to draw
        ReDim points(Main.MaxTemp - Main.MinTemp)

        'Draw the input equation results
        'Calculate the scaled points
        For k = 0 To UBound(points)
            points(k) = New Point(HScale * k + O, H + O + VOffset - (VScale * Main.Input(k + Main.MinTemp)))
        Next
        'Draw the points on the graph
        e.Graphics.DrawLines(greenPen, points)

        'Note the graph is scaled according to the values of the input equation. 
        'The output points may exceed the limits of the form.
        'If they exceed, they are simply drawn at the max or min value
        'Draw the output equation results
        'Calculate the scaled points
        For k = 0 To UBound(points)
            points(k) = New Point(HScale * k + O, H + O + VOffset - (VScale * Main.Output(0, k + Main.MinTemp)))
            If points(k).Y < O Then
                points(k).Y = O
            End If
        Next
        'draw the points on the graph
        e.Graphics.DrawLines(redPen, points)

        'Outline of the graph
        Dim OutlinePoints As Point() = {point1, point2, point3, point4, point5}

        'Draw lines between original points to screen.
        e.Graphics.DrawLines(bluePen, OutlinePoints)

        'Place text on the graph
        'Initialize the text properties
        Dim font As New Font("Times New Roman", 12, FontStyle.Bold, GraphicsUnit.Pixel)

        Dim pointT1 As New Point
        Dim pointT2 As New Point

        pointT1.Y = O + H + 5
        pointT2.X = O - 45

        'place the labels on the graph
        For k = 0 To 10
            pointT1.X = O + k * (W / 10) - 10
            pointT2.Y = O + H - k * (H / 10) - 10

            'X-Axis Labels
            GraphText = (Main.MinTemp) - 40 + k * ((Main.MaxTemp - Main.MinTemp) / 10)
            TextRenderer.DrawText(e.Graphics, GraphText, font, pointT1, Color.CadetBlue)

            'Y-Axis Labels
            GraphText = Format(Min + (k - 1) * 1.3 * (Max - Min) / 10, "0.00E+0")
            TextRenderer.DrawText(e.Graphics, GraphText, font, pointT2, Color.CadetBlue)
        Next

    End Sub

#End Region

End Class